# Publish Plugin Marker Artifacts

**Owners:**
Pepper Lebeck-Jobe [@eljobe](https://github.com/eljobe)
Stefan Oehme [@oehme](https://github.com/oehme)   
**Updated:** 2016-05-23   
![Approved](https://img.shields.io/badge/design-approved-green.svg)

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->


- [Objective](#objective)
  - [Goals / Use Cases](#goals--use-cases)
  - [Non Goals / Out of Scope Use Cases](#non-goals--out-of-scope-use-cases)
- [Solution](#solution)
- [Test Plan](#test-plan)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Objective

With the possibility to resolve from Maven and Ivy repositories, we
added a mapping to derive the Maven/Ivy coordinates from the plugin id
and version. The mapping is as follows:

| Plugin Datum          | Maven Concept | Ivy Concept |
| :-----------          | :------------ | :---------- |
| id                    | groupId       | org         |
| id + '.gradle.plugin' | artifactId    | name        |
| version               | version       | rev         |

This means that for every plugin contained in a project, there needs
to be a Maven/Ivy descriptor with the plugin’s coordinates and a
transitive dependency on the plugin implementation artifact. We call
this a "plugin marker". The goal of this milestone is to automate
publishing of these plugin markers.

### Goals / Use Cases

* As a plugin developer, I want to publish my plugins to a custom Maven/Ivy repository so that it can be resolved through the new `pluginRepositories`/`plugins` DSL

### Non Goals / Out of Scope Use Cases

* As a member of the infrastrcuture team, I want to mirror [plugins.gradle.org](https://plugins.gradle.org) as if it was a plain Maven repository. (This would require the plugin portal to also contain plugin markers)

## Solution

We integrate the `java-gradle-plugin` with the `maven-publish` and
`ivy-publish` plugin. For each plugin descriptor found in the project,
a plugin marker publication is added. The publication has a dependency
on the project’s main artifact using
`project.group:project.name:project.version`. A publication for the
main artifact is also created automatically, so that users only have
to specify their repositories to get started with publishing.

Currently, the plugin descriptors are discovered and validated only
while building the Jar of the project. We replace this with explicitly
declared plugins, using the following syntax:

```gradle
plugins {
  helloPlugin {
    id = 'com.example.hello'
    implementationClass = 'com.example.HelloPlugin'
  }
}
```

From this information, we generate publications and automate the
creation of the plugin descriptor files.

The marker publications use the name of the plugin as their
prefix. For Maven, it is suffixed with `MavenPluginMarker`, for Ivy
with `IvyPluginMarker` (the user might be publishing to both Maven and
Ivy, so we need to disambiguate). E.g. in the example above is a
publication called `helloPluginMavenPluginMarker`.

We add an opt-out flag `automatedPublishing`) to the
GradlePluginDevelopmentExtension. The feature is enabled by default.

## Test Plan

* If the maven-publish plugin is applied, the plugin implementation and a corresponding Maven marker are published
* If the ivy-publish plugin is applied, an the plugin implementation and a corresponding Ivy marker are published
* Publishing multiple plugins within a single project => one marker per plugin is added
* If the user sets automatedPublishing to false, no publications are added
* If the maven-publish plugin is not applied, no Maven plugin markers are added
* If the ivy-publish plugin is not applied, no Ivy plugin markers are added
* Declaring a plugin without an id or implementation class fails the build
* Plugin descriptor files are automatically generated and packaged for each declared plugin
* Undeclared plugins with hand-written descriptor files work as before, but are not included in publishing
